/***********************************************************************************************************************
 *
 * blueBill Resources - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://java.net/hg/bluebill~resources-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy.birds.ebn.italia;

import javax.annotation.Nonnull;
import java.util.Locale;
import java.util.Scanner;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryException;
import it.tidalwave.bluebill.taxonomy.Taxonomy;
import it.tidalwave.bluebill.taxonomy.Taxon;
import it.tidalwave.bluebill.taxonomy.birds.BirdLocaleInfo;
import it.tidalwave.bluebill.taxonomy.birds.BirdTaxonomyImporter;
import it.tidalwave.bluebill.taxonomy.birds.impl.PatchedDefaultMutableDisplayable;
import lombok.extern.slf4j.Slf4j;
import static it.tidalwave.bluebill.taxonomy.Taxon.Rank.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Slf4j
public class EbnItalia2011Importer extends BirdTaxonomyImporter
  {
    private static final String[] LANGUAGES = { "en", "it" };

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    class LanguageInitializer extends PatchedDefaultMutableDisplayable
      {
        public LanguageInitializer (final @Nonnull String[] languages,
                                    final @Nonnull String italianName,
                                    final @Nonnull String englishName,
                                    final @Nonnull BirdLocaleInfo localeInfo,
                                    final @Nonnull String scientificName)
          {
            for (final String language : languages)
              {
                String localizedName = "";

                // Italian and english are taken from the EBN checklist;
                if ("it".equals(language))
                  {
                    localizedName = italianName;
                  }
                else if ("en".equals(language))
                  {
                    localizedName = englishName;
                  }

                setDisplayName(localizedName, new Locale(language));
              }
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Taxonomy run (final @Nonnull Repository repository,
                         final @Nonnull String idPrefix)
      throws Exception
      {
        init(repository);

        final Taxonomy taxonomy = createTaxonomy(repository, idPrefix, EbnItalia2011Importer.class, LANGUAGES);
        final Taxon aves = createAves(taxonomy, idPrefix);
        
        final InputStream is = getClass().getResourceAsStream(sourceResourceName);
        final BufferedReader br = new BufferedReader(new InputStreamReader(is, "UTF-8"));
        process2011(br, taxonomy, aves, birdLocaleInfo, idPrefix);
        br.close();
        taxonUniqueIdManager.close();

        return taxonomy;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void process2011 (final @Nonnull BufferedReader br,
                              final @Nonnull Taxonomy taxonomy,
                              final @Nonnull Taxon aves,
                              final @Nonnull BirdLocaleInfo localeInfo,
                              final @Nonnull String idPrefix)
      throws IOException, RepositoryException
      {
        initialize();
        
        br.readLine();
        br.readLine();
        br.readLine();
        br.readLine();

        Taxon order = null;
        Taxon family = null;
        String orderPath = "";
        String familyPath = "";

        for (;;)
          {
            String s = br.readLine();

            if (s == null)
              {
                break;
              }

            s = s.trim();
            s = taxonTranslator.fixTypos(s);

            log.trace(">>>> {}", s);
            
            if (s.equals("") || s.equals("$$$$$$$$$$")) 
              {
                continue;
              }

            if (s.startsWith("\"segnando nella"))
              {
                break;
              }

            s = s.replace('$', '@').replaceAll("@@", "@ @");
            final Scanner scanner = new Scanner(s).useDelimiter("@");
//            scanner.next();
            final String italianName = scanner.next().replaceAll("\"", "").replaceAll("\\(.*\\)", "").trim();                
            final String scientificName = scanner.next().replaceAll("\"", "").trim();
            final String subSpeciesScientificName = scanner.next().replaceAll("\"", "").trim();
            final String subSpeciesItalianName = scanner.next().replaceAll("\"", "").trim();
            final String englishName = scanner.next().replaceAll("\"", "").trim();
            
//            log.trace("NAMES: *%s* *%s* *%s*", italianName, scientificName, subSpeciesItalianName);

            if (!subSpeciesItalianName.equals("") && scientificName.equals(""))
              {
                log.warn("Ignoring subspecies {} - {}", subSpeciesItalianName, subSpeciesScientificName);                        
              }
            else if (italianName.equals(""))
              {
                if (scientificName.endsWith("ES"))
                  {
                    final String orderName = capitalized(scientificName);
                    log.info("**************** ORDER: *{}*", orderName);
                    orderPath = "/Animalia/Chordata/Aves/" + orderName;
                    order = findOrCreateSubTaxon(aves, idPrefix, orderPath).withRank(ORDER).
                                                                            withScientificName(orderName).
                                                                            withSpecificEpythet(orderName).
                                                                            build();
                  }
                else
                  {  
                    final String familyName = scientificName;
                    log.info("**************** FAMILY: *{}*", familyName);
                    familyPath = orderPath + "/" + familyName;
                    family = findOrCreateSubTaxon(order, idPrefix, familyPath).withRank(FAMILY).
                                                                               withScientificName(familyName).
                                                                               withSpecificEpythet(familyName).
                                                                               build();
                  }
              } 
            else
              {
                final Scanner scanner2 = new Scanner(scientificName).useDelimiter(" ");
                final String genusName = scanner2.next();
                final String speciesName = scanner2.next();

                final String genusPath = familyPath + "/" + genusName;
                final String speciesPath = genusPath + "/" + speciesName;

                final LanguageInitializer commonNames = new LanguageInitializer(LANGUAGES, italianName, englishName, localeInfo, scientificName);
                final Taxon genus = findOrCreateSubTaxon(family, idPrefix, genusPath).withRank(GENUS).
                                                                                      withScientificName(genusName).
                                                                                      withSpecificEpythet(genusName).
                                                                                      build();
                final Taxon species = findOrCreateSubTaxon(genus, idPrefix, speciesPath).withRank(SPECIES).
                                                                                         withScientificName(genusName + " " + speciesName).
                                                                                         withSpecificEpythet(speciesName).
                                                                                         withDisplayNames(commonNames).
                                                                                         build();
              }
          }
      }
  }
